import tempfile
import unittest

import PIL.Image

import pillowfight


class TestAll(unittest.TestCase):
    def test_all_1(self):
        with tempfile.NamedTemporaryFile(suffix='.jpg') as tmpfile:
            in_img = PIL.Image.open("tests/data/black_border_problem.jpg")

            out_img = pillowfight.ace(in_img, seed=0xDEADBEE)

            # unpaper order
            out_img = pillowfight.unpaper_blackfilter(out_img)
            out_img = pillowfight.unpaper_noisefilter(out_img)
            out_img = pillowfight.unpaper_blurfilter(out_img)
            out_img = pillowfight.unpaper_masks(out_img)
            out_img = pillowfight.unpaper_grayfilter(out_img)
            out_img = pillowfight.unpaper_border(out_img)

            in_img.close()

            # beware of JPG compression
            self.assertEqual(out_img.mode, "RGB")
            out_img.save(tmpfile.name)
            out_img.close()
            out_img = PIL.Image.open(tmpfile.name)

        expected_img = PIL.Image.open(
            "tests/data/black_border_problem_all.jpg"
        )
        self.assertEqual(out_img.tobytes(), expected_img.tobytes())
        expected_img.close()

    def test_all_2(self):
        with tempfile.NamedTemporaryFile(suffix='.jpg') as tmpfile:
            in_img = PIL.Image.open("tests/data/brightness_problem.jpg")

            out_img = pillowfight.ace(in_img, seed=0xBEEDEAD)

            # unpaper order
            out_img = pillowfight.unpaper_blackfilter(out_img)
            out_img = pillowfight.unpaper_noisefilter(out_img)
            out_img = pillowfight.unpaper_blurfilter(out_img)
            out_img = pillowfight.unpaper_masks(out_img)
            out_img = pillowfight.unpaper_grayfilter(out_img)
            out_img = pillowfight.unpaper_border(out_img)

            in_img.close()

            # beware of JPG compression
            self.assertEqual(out_img.mode, "RGB")
            out_img.save(tmpfile.name)
            out_img.close()
            out_img = PIL.Image.open(tmpfile.name)

        expected_img = PIL.Image.open(
            "tests/data/brightness_problem_all.jpg"
        )
        self.assertEqual(out_img.tobytes(), expected_img.tobytes())
        expected_img.close()



import tempfile
import unittest

import PIL.Image

import pillowfight


class TestGrayFilter(unittest.TestCase):
    def test_grayfilter(self):
        with tempfile.NamedTemporaryFile(suffix='.jpg') as tmpfile:
            in_img = PIL.Image.open("tests/data/black_border_problem.jpg")
            out_img = pillowfight.unpaper_grayfilter(in_img)
            in_img.close()

            # beware of JPG compression
            self.assertEqual(out_img.mode, "RGB")
            out_img.save(tmpfile.name)
            out_img.close()
            out_img = PIL.Image.open(tmpfile.name)

        expected_img = PIL.Image.open(
            "tests/data/black_border_problem_grayfilter.jpg"
        )
        self.assertEqual(out_img.tobytes(), expected_img.tobytes())
        expected_img.close()



#!/usr/bin/env python3

import os
import sys
import urllib.error
import urllib.request

import PIL
import PIL.Image


SCAN_URL_FMT = "https://openpaper.work/scannerdb/report/{id}/scanned.png"


def main():
    min_report_id = int(sys.argv[1])
    max_report_id = int(sys.argv[2])
    out_dir = sys.argv[3]

    os.makedirs(out_dir, exist_ok=True)

    for report_id in range(min_report_id, max_report_id + 1):
        print("")
        scan = SCAN_URL_FMT.format(id=report_id)
        print(f"Downloading {scan} ...")
        try:
            scan = urllib.request.urlopen(scan)
        except Exception as exc:
            print("Failed: {}".format(str(exc)))
            continue
        code = scan.getcode()
        print(f"Reply: {code}")
        if code != 200:
            continue

        img = PIL.Image.open(scan)
        print(f"Image size: {img.size}")
        if img.size[0] < 256 or img.size[1] < 256:
            print("Too small")
            continue

        img.save(os.path.join(out_dir, f"in_{report_id}.jpeg"))
        print("Done")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import sys

import PIL.Image
import PIL.ImageDraw

import pillowfight


def find_scan_borders(img_in, img_out):
    img_in = PIL.Image.open(img_in)

    img = img_in.copy()
    # img = pillowfight.unpaper_blackfilter(img)
    # img = pillowfight.unpaper_noisefilter(img)
    # img = pillowfight.unpaper_blurfilter(img)
    # img = pillowfight.unpaper_masks(img)

    frame = pillowfight.find_scan_borders(img)

    draw = PIL.ImageDraw.Draw(img_in)
    draw.rectangle(
        ((0, 0), (frame[0], img_in.size[1])),
        fill=(0xc4, 0x00, 0xff)
    )
    draw.rectangle(
        ((0, 0), (img_in.size[0], frame[1])),
        fill=(0xc4, 0x00, 0xff)
    )
    draw.rectangle(
        ((frame[2], 0), (img_in.size[0], img_in.size[1])),
        fill=(0xc4, 0x00, 0xff)
    )
    draw.rectangle(
        ((0, frame[3]), (img_in.size[0], img_in.size[1])),
        fill=(0xc4, 0x00, 0xff)
    )
    img_in.save(img_out)


def main():
    find_scan_borders(sys.argv[1], sys.argv[2])


if __name__ == "__main__":
    main()



import unittest

import PIL.Image
import PIL.ImageDraw

import pillowfight


class TestScanBorders(unittest.TestCase):
    def test_scan_borders_a(self):
        in_img = PIL.Image.open("tests/data/brother_mfc7360.jpeg")
        frame = pillowfight.find_scan_borders(in_img)
        self.assertEqual(frame, (8, 37, 1637, 2195))

    def test_scan_borders_b(self):
        in_img = PIL.Image.open("tests/data/epson_xp425.jpeg")
        frame = pillowfight.find_scan_borders(in_img)
        self.assertEqual(frame, (11, 4, 2541, 3503))

    def test_scan_borders_c(self):
        in_img = PIL.Image.open("tests/data/brother_ds620.jpeg")
        frame = pillowfight.find_scan_borders(in_img)
        self.assertEqual(frame, (3, 3, 2507, 3527))



#!/usr/bin/env python3

import multiprocessing.pool
import os
import sys

import PIL.Image
import PIL.ImageDraw

import pillowfight


SCAN_URL_FMT = "https://openpaper.work/scannerdb/report/{id}/scanned.png"


count = 0


def _find_scan_borders(img_in):
    img_in = img_in.copy()
    frame = pillowfight.find_scan_borders(img_in)

    draw = PIL.ImageDraw.Draw(img_in)
    draw.rectangle(
        ((0, 0), (frame[0], img_in.size[1])),
        fill=(0xc4, 0x00, 0xff)
    )
    draw.rectangle(
        ((0, 0), (img_in.size[0], frame[1])),
        fill=(0xc4, 0x00, 0xff)
    )
    draw.rectangle(
        ((frame[2], 0), (img_in.size[0], img_in.size[1])),
        fill=(0xc4, 0x00, 0xff)
    )
    draw.rectangle(
        ((0, frame[3]), (img_in.size[0], img_in.size[1])),
        fill=(0xc4, 0x00, 0xff)
    )
    return img_in


def find_scan_borders(report_id, img_in, total):
    global count

    progress = int(20 * (count / total))
    progress = ("=" * progress) + (" " * (20 - progress))
    sys.stdout.write(f"\r[{progress}] {count}/{total}")
    sys.stdout.flush()

    img_in = img_in.convert("RGB")
    img_out = _find_scan_borders(img_in)

    count += 1
    return (report_id, img_in, img_out)


def main():
    global count

    in_dir = sys.argv[1]
    out_dir = sys.argv[2]

    os.makedirs(out_dir, exist_ok=True)

    pool = multiprocessing.pool.ThreadPool(
        multiprocessing.cpu_count()
    )

    print("Loading ...")
    all_files = list(os.listdir(in_dir))

    for x in range(0, len(all_files), 64):
        imgs = []

        for scan in all_files[x:x + 64]:
            if not scan.startswith("in_"):
                continue
            report_id = int(scan[len("in_"):-len(".jpeg")])

            img_in = PIL.Image.open(os.path.join(in_dir, scan))
            imgs.append((report_id, img_in))

        print("Processing ...")
        imgs = pool.map(
            lambda args: find_scan_borders(args[0], args[1], len(all_files)),
            imgs
        )

        print()
        print("Writing ...")
        for (report_id, img_in, img_out) in imgs:
            assembly = [img_in, img_out]
            (widths, heights) = zip(*(i.size for i in assembly))
            total_width = sum(widths)
            max_height = max(heights)

            img = PIL.Image.new('RGB', (total_width, max_height))
            x_offset = 0
            for i in assembly:
                img.paste(i, (x_offset, 0))
                x_offset += i.size[0]
            img.save(os.path.join(out_dir, f"out_{report_id}.jpeg"))

    print("Done")


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import os
import platform
import sys

from setuptools import Extension, setup

if os.name == "nt":
    libdep = []
    extra_compile_args = []
else:
    libdep = ["m", "pthread"]
    if platform.processor() == "i386":
        extra_compile_args = ['-msse2', '-mfpmath=sse']
    else:
        extra_compile_args = []

try:
    with open("src/pillowfight/_version.h", "r") as file_descriptor:
        version = file_descriptor.read().strip()
        version = version.split(" ")[2][1:-1]
        if "-" in version:
            version = version.split("-")[0]
except FileNotFoundError:
    print("WARNING: version.txt file is missing")
    print("WARNING: Please run 'make version' first")
    sys.exit(1)


setup(
    name="pypillowfight",
    version=version,
    description=("Library containing various image processing algorithms"),
    long_description=("Library containing various image processing algorithms:"
                      " Automatic Color Equalization, Unpaper's algorithms,"
                      " Stroke Width Transformation, etc"),
    keywords="image processing algorithm pillow pil",
    url="https://gitlab.gnome.org/World/OpenPaperwork/libpillowfight#readme",
    download_url=(
        "https://gitlab.gnome.org/World/OpenPaperwork/libpillowfight/archive/"
        "{}.zip".format(version)
    ),
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Topic :: Multimedia :: Graphics :: Graphics Conversion",
    ],
    license="GPLv2",
    author="Jerome Flesch",
    author_email="jflesch@openpaper.work",
    packages=[
        'pillowfight',
    ],
    package_dir={
        'pillowfight': 'src/pillowfight',
    },
    ext_modules=[
        Extension(
            'pillowfight._clib', [
                'src/pillowfight/util.c',
                'src/pillowfight/_ace.c',
                'src/pillowfight/_blackfilter.c',
                'src/pillowfight/_blurfilter.c',
                'src/pillowfight/_border.c',
                'src/pillowfight/_canny.c',
                'src/pillowfight/_compare.c',
                'src/pillowfight/_gaussian.c',
                'src/pillowfight/_grayfilter.c',
                'src/pillowfight/_masks.c',
                'src/pillowfight/_noisefilter.c',
                'src/pillowfight/_pymod.c',
                'src/pillowfight/_scanborders.c',
                'src/pillowfight/_sobel.c',
                'src/pillowfight/_swt.c',
                'src/pillowfight/_version.c',
            ],
            include_dirs=["include"],
            libraries=libdep,
            extra_compile_args=extra_compile_args,
            undef_macros=['NDEBUG'],
        ),
    ],
    data_files=[],
    scripts=[],
    install_requires=[
        "Pillow",
    ],
    setup_requires=['nose>=1.0'],
)
